﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace HeuristicLab.Problems.Instances.Regression {
  public abstract class RegressionInstanceProvider : IProblemInstanceProvider<RegressionData> {
    public RegressionData LoadData(string path) {
      TableFileParser csvFileParser = new TableFileParser();
      csvFileParser.Parse(path);

      RegressionData regData = new RegressionData();
      int pos = path.LastIndexOf('\\');
      if (pos < 0)
        regData.Name = path;
      else {
        pos++;
        regData.Name = path.Substring(pos, path.Length - pos);
      }
      regData.InputVariables = new List<string>(csvFileParser.VariableNames);
      regData.TargetVariable = csvFileParser.VariableNames.Last();
      regData.AllowedInputVariables = regData.InputVariables.Where(x => !x.Equals(regData.TargetVariable));
      //convert to multidimensional array
      List<IList> values = csvFileParser.Values;
      regData.Values = new double[values.First().Count, values.Count];
      for (int i = 0; i < values.Count; i++) {
        for (int j = 0; j < values.First().Count; j++) {
          regData.Values[j, i] = (double)values[i][j];
        }
      }

      int trainingPartEnd = csvFileParser.Rows * 2 / 3;
      regData.TrainingPartitionStart = 0;
      regData.TrainingPartitionEnd = trainingPartEnd;
      regData.TestPartitionStart = trainingPartEnd;
      regData.TestPartitionEnd = csvFileParser.Rows;
      return regData;
    }

    public void SaveData(RegressionData instance, string path) {
      StringBuilder strBuilder = new StringBuilder();

      foreach (var variable in instance.InputVariables) {
        strBuilder.Append(variable + ";");
      }
      strBuilder.Remove(strBuilder.Length - 1, 1);
      strBuilder.AppendLine();

      double[,] values = instance.Values;

      for (int i = 0; i < values.GetLength(0); i++) {
        for (int j = 0; j < values.GetLength(1); j++) {
          strBuilder.Append(values[i, j] + ";");
        }
        strBuilder.Remove(strBuilder.Length - 1, 1);
        strBuilder.AppendLine();
      }

      using (StreamWriter writer = new StreamWriter(path)) {
        writer.Write(strBuilder);
      }
    }

    public abstract IEnumerable<IDataDescriptor> GetDataDescriptors();
    public abstract RegressionData LoadData(IDataDescriptor descriptor);

    public abstract string Name { get; }
    public abstract string Description { get; }
    public abstract Uri WebLink { get; }
    public abstract string ReferencePublication { get; }

    public IProblemInstanceConsumer<RegressionData> Consumer { get; set; }
  }
}
