﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.IO;
using System.Linq;
using System.Reflection;

namespace HeuristicLab.Problems.Instances.Regression {
  public abstract class ResourceRegressionInstanceProvider : RegressionInstanceProvider {
    public override RegressionData LoadData(IDataDescriptor id) {
      var descriptor = (ResourceRegressionDataDescriptor)id;

      RegressionData regData = LoadData(GetTempFileForResource(descriptor.resourceName));
      regData.Name = descriptor.Name;
      regData.Description = descriptor.Description;

      return regData;
    }

    #region Helpers
    protected static string GetTempFileForResource(string resourceName) {
      Assembly assembly = Assembly.GetExecutingAssembly();
      string resource = assembly.GetManifestResourceNames().Where(x => x.EndsWith(resourceName)).First();

      string path = Path.GetTempFileName();

      using (Stream stream = assembly.GetManifestResourceStream(resource)) {
        WriteStreamToTempFile(stream, path);
      }
      return path;
    }

    private static void WriteStreamToTempFile(Stream stream, string path) {
      using (FileStream output = new FileStream(path, FileMode.Create, FileAccess.Write)) {
        int cnt = 0;
        byte[] buffer = new byte[32 * 1024];
        while ((cnt = stream.Read(buffer, 0, buffer.Length)) != 0)
          output.Write(buffer, 0, cnt);
      }
    }
    #endregion
  }
}
